/*

    FFT filter 2
    Copyright (C) 1999-2001 Jussi Laako

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "dsp/DSPConfig.hh"
#include "dsp/DSPOp.hh"
#ifndef DSP_USE_DEQUE
#include "dsp/ReBuffer.hh"
#else
#include "dsp/ReBuffer2.hh"
#endif


#ifndef FILTER2_HH
    #define FILTER2_HH


    /**
        Class implementing FFT-based FIR-filter with design functions.
        Filtering is done in complex-plane.
    */
    class clFilter2 : private clDSPOp
    {
            bool bInitialized;
            long lFFTSize;
            long lHalfSize;
            long lSpectPoints;
            clDSPAlloc Win;
            clDSPAlloc InvWin;
            clDSPAlloc CoeffWin;
            clDSPAlloc Prev;
            clDSPAlloc Proc;
            clDSPAlloc CCoeffs;
            clDSPAlloc CProc;
            #ifndef DSP_USE_DEQUE
            clReBuffer InBuf;
            clReBuffer OutBuf;
            #else
            clReBuffer2 InBuf;
            clReBuffer2 OutBuf;
            #endif
            /**
                Initialize complex coefficients.

                \f[\left\vert\begin{array}{l}\Re_{C}(i)=1\\
                \Im_{C}(i)=0\end{array}\right.\f]
            */
            void InitCoeffsS ();
            /// \overload
            void InitCoeffsD ();
            /**
                Create complex impulse response from specified frequency
                response.

                This is done by doing \f$FFT^{-1}(X)\f$, then multiplying
                it with desired window function and doing \f$FFT(x)\f$.
            */
            void ReadyFilterS ();
            /// \overload
            void ReadyFilterD ();
        public:
            clFilter2 ();
            ~clFilter2 ();
            /**
                Initialize filter.

                \param WindowSize Number of FFT points used in filter
                \param FiltCoeffs Optional coefficient vector, can be NULL
            */
            bool Initialize (long, const float *);
            /// \overload
            bool Initialize (long, const double *);
            /// \overload
            bool InitFloat (long);
            /// \overload
            bool InitDouble (long);
            /**
                Uninitialize filter.
            */
            void Uninitialize ();
            /**
                Set coefficients.

                You can feed the filter with desired frequency response.
                Length of this dataset is specified window size / 2 + 1.
                Filter is then created from this coefficient vector.

                \param FiltCoeffs Coefficients for designing filter
            */
            void SetCoeffs (const float *);
            /// \overload
            void SetCoeffs (const double *);
            /**
                Get coefficients.

                Get filter's frequency response.

                \param FiltCoeffs Filter coefficients
            */
            void GetCoeffs (float *);
            /// \overload
            void GetCoeffs (double *);
            /**
                Feed data into filter.

                Filtering is done by first multiplying data with optional
                window function, then doing \f$FFT(x)\f$, then doing complex
                multiply with complex impulse response, then doing
                \f$FFT^{-1}(X)\f$ and then multiplying with inverted optional
                window function. Windows overlap 50% and 50% of window length
                is used.

                \param SrcData Source data
                \param SrcCount Source data count
            */
            void Put (const float *, long);
            /// \overload
            void Put (const double *, long);
            /**
                Feed data into filter using complex coefficients.

                \param SrcData Source data
                \param SrcCount Source data count
                \param Coeffs Complex coefficients
            */
            void Put (const float *, long, const stpSCplx);
            /// \overload
            void Put (const double *, long, const stpDCplx);
            /**
                Get data from filter.

                \param DestData Destination buffer
                \param DestCount Number of samples
                \return Data was available?
            */
            bool Get (float *, long);
            /// \overload
            bool Get (double *, long);
            /**
                Design low-pass filter.

                \param Corner Corner frequency
                \param SampleRate Sampling frequency
                \param DCBlock Include DC-blocking
            */
            void DesignLP (float *, bool = false);
            /// \overload
            void DesignLP (double *, bool = false);
            /// \overload
            void DesignLP (float *, float, bool = false);
            /// \overload
            void DesignLP (double *, double, bool = false);
            /**
                Design high-pass filter.
            */
            void DesignHP (float *);
            /// \overload
            void DesignHP (double *);
            /// \overload
            void DesignHP (float *, float);
            /// \overload
            void DesignHP (double *, double);
            /**
                Design band-pass filter.

                \param LowCorner Lower corner frequency
                \param HighCorner Higher corner frequency
                \param SampleRate Sampling frequency
            */
            void DesignBP (float *, float *);
            /// \overload
            void DesignBP (double *, double *);
            /// \overload
            void DesignBP (float *, float *, float);
            /// \overload
            void DesignBP (double *, double *, double);
            /**
                Design band-reject filter.
            */
            void DesignBR (float *, float *);
            /// \overload
            void DesignBR (double *, double *);
            /// \overload
            void DesignBR (float *, float *, float);
            /// \overload
            void DesignBR (double *, double *, double);
    };

#endif

